==> TreeTools_0.1.2.tar.gz: TreeTools/R/TreeNumber.R <==
#' Unique integer indices for bifurcating tree topologies
#'
#' Functions converting between phylogenetic trees and their unique decimal
#' representation.
#'
#' There are `NUnrooted(n)` unrooted trees with _n_ tips.
#' As such, each _n_-tip tree can be uniquely identifyed by a non-negative
#' integer _x_ < `NUnrooted(n)`.
#'
#' This integer can be converted by a tree by treating it as a mixed-base number,
#' with bases 1, 3, 5, 7, ... (2_n_ - 5).
#'
#' Each digit of this mixed base number corresponds to a tip, and determines
#' the location on a growing tree to which that tip should be added.
#'
#' We start with a two-tip tree, and treat 0 as the origin of the tree.
#'
#' ```
#'
#' 0 ---- 1
#'
#' ```
#'
#' We add tip 2 by breaking an edge and inserting a node (numbered 2 + nTip - 1).
#' In this example, we'll work up to a six-tip tree; this node will be numbered
#' 2 + 6 - 1 = 7.
#' There is only one edge on which tip 2 can be added.  Let's add node 7 and tip 2:
#'
#' ```
#'
#' 0 ---- 7 ---- 1
#'        |
#'        |
#'        2
#'
#' ```
#'
#' There are now three edges on which tip 3 can be added.  Our options are:
#' Option 0: the edge leading to 1;
#' Option 1: the edge leading to 2;
#' Option 2: the edge leading to 7.
#'
#' If we select option 1, we produce:
#'
#' ```
#'
#' 0 ---- 7 ---- 1
#'        |
#'        |
#'        8 ---- 2
#'        |
#'        |
#'        3
#'
#' ```
#' `1` is now the final digit of our mixed-base number
#'
#' There are five places to add tip 4:
#' Option 0: the edge leading to 1;
#' Option 1: the edge leading to 2;
#' Option 2: the edge leading to 3;
#' Option 3: the edge leading to 7;
#' Option 4: the edge leading to 8.
#'
#' If we chose option 3, then `3` would be the penultimate digit of our
#' mixed-base number
#'
#' If we chose option 0 for the next two additions, we could specify this tree
#' with the mixed-base number 0021.  We can convert this into decimal:
#'
#' 0 &times; (1 &times; 3 &times; 5 &times; 9) +
#'
#' 0 &times; (1 &times; 3 &times; 5) +
#'
#' 3 &times; (1 &times; 3) +
#'
#' 1 &times; (1)
#'
#' = 10
#'
#' Note that the hyperexponential nature of tree space means that there are &gt;
#' 2^30 unique 12-tip trees.  As integers &gt; 2^31 are not supported by R,
#' numbers representing larger trees are represented internally as a vector of
#' nine-digit integer 'chunks' and passed to the underlying C code, where they
#' are combined into a single 64-bit integer.  This allows trees with up to
#' 42 tips to be accommodated.
#'
#' @param x Integer identifying the tree (see details).
#' @param nTip Integer specifying number of tips in the tree.
#' @template tipLabelsParam
#' @param \dots Additional parameters for consistency with S3 methods (unused).
#'
#' @return `as.phylo.numeric` returns a tree of class `phylo`.
#'
#' @examples
#' tree <- as.phylo(10, nTip = 6)
#' plot(tree)
#' as.TreeNumber(tree)
#'
#' # Larger trees:
#' as.TreeNumber(BalancedTree(19))
#'
#' # If > 9 digits, represent the tree number as a string.
#' treeNumber <- as.TreeNumber("1234567890123", nTip = 14)
#' tree <- as.phylo(treeNumber)
#'
#' @seealso [`TreeShape`]
#' @family tree generation functions
#' @exportClass TreeNumber
#' @name TreeNumber
#

#' @rdname TreeNumber
#'
#' @return `as.TreeNumber` returns an object of class `TreeNumber`,
#' which comprises a numeric vector, whose elements represent successive
#' nine-digit chunks of the decimal integer corresponding to the tree topology
#' (in big endian order).  The `TreeNumber` object has attributes
#' `nTip` and `tip.labels`.
#' @export
as.TreeNumber <- function(x, ...) UseMethod('as.TreeNumber')

#' @rdname TreeNumber
#' @importFrom ape root
#' @export
as.TreeNumber.phylo <- function (x, ...) {
  x <- root(x, 1, resolve.root = TRUE)
  edge <- x$edge
  nTip <- NTip(x)
  edge <- PostorderEdges(edge)
  structure(edge_to_num(edge[, 1], edge[, 2], nTip),
            nTip = nTip,
            tip.labels = TipLabels(x),
            class = 'TreeNumber')
}

#' @rdname TreeNumber
#' @export
as.TreeNumber.multiPhylo <- function (x, ...) {
  vapply(x, as.TreeNumber.phylo, structure(0, nTip = 0,
                                           tip.labels = '0',
                                           class = 'TreeNumber'))
}

#' @rdname TreeNumber
#' @export
as.TreeNumber.character <- function (x, nTip, tipLabels = TipLabels(nTip), ...) {
  len <- nchar(x)
  ends <- rev(len - (seq_len((len / 9L) + 1L) - 1L) * 9L)
  starts <- pmax(1L, ends - 8L)

  # Return:
  structure(as.integer(substring(x, starts, ends)),
            nTip = nTip,
            tip.labels = tipLabels,
            class = 'TreeNumber')
}

#' @rdname TreeNumber
#' @template MRS
#' @references Based on a concept by John Tromp, employed in Li _et al._ 1996.
#'
#' \insertRef{Li1996}{TreeTools}
#'
#' @examples
#' as.phylo(0:2, nTip = 6, tipLabels = letters[1:6])
#'
#' @importFrom ape as.phylo
#' @export
as.phylo.numeric <- function (x, nTip = attr(x, 'nTip'),
                              tipLabels = attr(x, 'tip.label'), ...) {
  if (is.null(tipLabels)) tipLabels <- paste0('t', seq_len(nTip))
  if (length(x) > 1) {
    structure(lapply(x, as.phylo.numeric, nTip = nTip, tipLabels = tipLabels),
              tip.label = tipLabels, class='multiPhylo')
  } else {
    edge <- RenumberEdges(num_to_parent(x, nTip), seq_len(nTip + nTip - 2L))
    structure(list(edge = do.call(cbind, edge),
                   tip.label = tipLabels,
                   Nnode = nTip - 1L),
              order = 'preorder',
              class = 'phylo')
  }
}

#' @rdname TreeNumber
#' @export
as.phylo.TreeNumber <- function (x, nTip = attr(x, 'nTip'),
                                 tipLabels = attr(x, 'tip.label'), ...) {
  if (is.null(tipLabels)) tipLabels <- paste0('t', seq_len(nTip))
  edge <- RenumberEdges(num_to_parent(x, nTip), seq_len(nTip + nTip - 2L))
  structure(list(edge = do.call(cbind, edge),
                 tip.label = tipLabels,
                 Nnode = nTip - 1L),
            order = 'preorder',
            class = 'phylo')
}

#' Print TreeNumber object
#'
#' S3 method for objects of class `TreeNumber`.
#'
#' @param x Object of class `TreeNumber`.
#' @param \dots Additional arguments for consistency with S3 method (unused).
#'
#' @export
print.TreeNumber <- function (x, ...) {
  nTip <- attr(x, 'nTip')
  cat("Phylogenetic tree number", .PrintedTreeNumber(x), "of", NUnrooted(nTip),
      "\n", nTip, "tips:", paste0(attr(x, 'tip.labels')))
}

#' @keywords internal
.PrintedTreeNumber <- function (x) {
  paste0(c(x[1], formatC(x[-1], digits = 9L, flag='0')), collapse = '')
}

==> TreeTools_0.1.4.tar.gz: TreeTools/R/TreeNumber.R <==
#' Unique integer indices for bifurcating tree topologies
#'
#' Functions converting between phylogenetic trees and their unique decimal
#' representation.
#'
#' There are `NUnrooted(n)` unrooted trees with _n_ leaves.
#' As such, each _n_-leaf tree can be uniquely identified by a non-negative
#' integer _x_ < `NUnrooted(n)`.
#'
#' This integer can be converted by a tree by treating it as a mixed-base number,
#' with bases 1, 3, 5, 7, ... (2_n_ - 5).
#'
#' Each digit of this mixed base number corresponds to a leaf, and determines
#' the location on a growing tree to which that leaf should be added.
#'
#' We start with a two-leaf tree, and treat 0 as the origin of the tree.
#'
#' ```
#'
#' 0 ---- 1
#'
#' ```
#'
#' We add leaf 2 by breaking an edge and inserting a node (numbered
#' `2 + nTip - 1`).
#' In this example, we'll work up to a six-leaf tree; this node will be numbered
#' 2 + 6 - 1 = 7.
#' There is only one edge on which leaf 2 can be added.  Let's add node 7 and
#' leaf 2:
#'
#' ```
#'
#' 0 ---- 7 ---- 1
#'        |
#'        |
#'        2
#'
#' ```
#'
#' There are now three edges on which leaf 3 can be added.  Our options are:
#' Option 0: the edge leading to 1;
#' Option 1: the edge leading to 2;
#' Option 2: the edge leading to 7.
#'
#' If we select option 1, we produce:
#'
#' ```
#'
#' 0 ---- 7 ---- 1
#'        |
#'        |
#'        8 ---- 2
#'        |
#'        |
#'        3
#'
#' ```
#' `1` is now the final digit of our mixed-base number
#'
#' There are five places to add leaf 4:
#' Option 0: the edge leading to 1;
#' Option 1: the edge leading to 2;
#' Option 2: the edge leading to 3;
#' Option 3: the edge leading to 7;
#' Option 4: the edge leading to 8.
#'
#' If we chose option 3, then `3` would be the penultimate digit of our
#' mixed-base number
#'
#' If we chose option 0 for the next two additions, we could specify this tree
#' with the mixed-base number 0021.  We can convert this into decimal:
#'
#' 0 &times; (1 &times; 3 &times; 5 &times; 9) +
#'
#' 0 &times; (1 &times; 3 &times; 5) +
#'
#' 3 &times; (1 &times; 3) +
#'
#' 1 &times; (1)
#'
#' = 10
#'
#' Note that the hyperexponential nature of tree space means that there are &gt;
#' 2^30 unique 12-leaf trees.  As integers &gt; 2^31 are not supported by R,
#' numbers representing larger trees are represented internally as a vector of
#' nine-digit integer 'chunks' and passed to the underlying C code, where they
#' are combined into a single 64-bit integer.  This allows trees with up to
#' 42 leaves to be accommodated.
#'
#' @param x Integer identifying the tree (see details).
#' @param nTip Integer specifying number of leaves in the tree.
#' @template tipLabelsParam
#' @param \dots Additional parameters for consistency with S3 methods (unused).
#'
#' @return `as.phylo.numeric` returns a tree of class `phylo`.
#'
#' @examples
#' tree <- as.phylo(10, nTip = 6)
#' plot(tree)
#' as.TreeNumber(tree)
#'
#' # Larger trees:
#' as.TreeNumber(BalancedTree(19))
#'
#' # If > 9 digits, represent the tree number as a string.
#' treeNumber <- as.TreeNumber("1234567890123", nTip = 14)
#' tree <- as.phylo(treeNumber)
#'
#' @seealso [`TreeShape`]
#' @family tree generation functions
#' @exportClass TreeNumber
#' @name TreeNumber
#

#' @rdname TreeNumber
#'
#' @return `as.TreeNumber` returns an object of class `TreeNumber`,
#' which comprises a numeric vector, whose elements represent successive
#' nine-digit chunks of the decimal integer corresponding to the tree topology
#' (in big endian order).  The `TreeNumber` object has attributes
#' `nTip` and `tip.labels`.
#' @export
as.TreeNumber <- function(x, ...) UseMethod('as.TreeNumber')

#' @rdname TreeNumber
#' @importFrom ape root
#' @export
as.TreeNumber.phylo <- function (x, ...) {
  x <- root(x, 1, resolve.root = TRUE)
  edge <- x$edge
  nTip <- NTip(x)
  edge <- PostorderEdges(edge)
  structure(edge_to_num(edge[, 1], edge[, 2], nTip),
            nTip = nTip,
            tip.labels = TipLabels(x),
            class = 'TreeNumber')
}

#' @rdname TreeNumber
#' @export
as.TreeNumber.multiPhylo <- function (x, ...) {
  vapply(x, as.TreeNumber.phylo, structure(0, nTip = 0,
                                           tip.labels = '0',
                                           class = 'TreeNumber'))
}

#' @rdname TreeNumber
#' @export
as.TreeNumber.character <- function (x, nTip, tipLabels = TipLabels(nTip), ...) {
  len <- nchar(x)
  ends <- rev(len - (seq_len((len / 9L) + 1L) - 1L) * 9L)
  starts <- pmax(1L, ends - 8L)

  # Return:
  structure(as.integer(substring(x, starts, ends)),
            nTip = nTip,
            tip.labels = tipLabels,
            class = 'TreeNumber')
}

#' @rdname TreeNumber
#' @template MRS
#' @references Based on a concept by John Tromp, employed in Li _et al._ 1996.
#'
#' \insertRef{Li1996}{TreeTools}
#'
#' @examples
#' as.phylo(0:2, nTip = 6, tipLabels = letters[1:6])
#'
#' @importFrom ape as.phylo
#' @export
as.phylo.numeric <- function (x, nTip = attr(x, 'nTip'),
                              tipLabels = attr(x, 'tip.label'), ...) {
  if (is.null(nTip)) {
    if (is.null(tipLabels)) {
      stop("Either nTip or tipLabels must be specified.")
    } else {
      nTip <- length(tipLabels)
    }
  }
  if (is.null(tipLabels)) tipLabels <- paste0('t', seq_len(nTip))
  if (length(x) > 1) {
    structure(lapply(x, as.phylo.numeric, nTip = nTip, tipLabels = tipLabels),
              tip.label = tipLabels, class='multiPhylo')
  } else {
    edge <- RenumberEdges(num_to_parent(x, nTip), seq_len(nTip + nTip - 2L))
    structure(list(edge = do.call(cbind, edge),
                   tip.label = tipLabels,
                   Nnode = nTip - 1L),
              order = 'preorder',
              class = 'phylo')
  }
}

#' @rdname TreeNumber
#' @export
as.phylo.TreeNumber <- function (x, nTip = attr(x, 'nTip'),
                                 tipLabels = attr(x, 'tip.label'), ...) {
  if (is.null(tipLabels)) tipLabels <- paste0('t', seq_len(nTip))
  edge <- RenumberEdges(num_to_parent(x, nTip), seq_len(nTip + nTip - 2L))
  structure(list(edge = do.call(cbind, edge),
                 tip.label = tipLabels,
                 Nnode = nTip - 1L),
            order = 'preorder',
            class = 'phylo')
}

#' Print `TreeNumber` object
#'
#' S3 method for objects of class `TreeNumber`.
#'
#' @param x Object of class `TreeNumber`.
#' @param \dots Additional arguments for consistency with S3 method (unused).
#'
#' @export
print.TreeNumber <- function (x, ...) {
  nTip <- attr(x, 'nTip')
  cat("Phylogenetic tree number", .PrintedTreeNumber(x), "of", NUnrooted(nTip),
      "\n", nTip, "tips:", paste0(attr(x, 'tip.labels')))
}

#' @keywords internal
.PrintedTreeNumber <- function (x) {
  paste0(c(x[1], formatC(x[-1], digits = 9L, flag='0')), collapse = '')
}
